keys = {}
keys.version = "Version 0.8"

keys.running = false

keys.infos = {}

keys.start_timer = Timer()
keys.refresh_key_info_timer = Timer()
keys.refresh_ui_timer = Timer()

keys.ui = {}
keys.ui.stations = {}
keys.ui.manager = {}
keys.ui.copier = {}

dofile('ui-stations.lua')
dofile('ui-manager.lua')
dofile('ui-copier.lua')
dofile('ui.lua')

function keys.remove_color(s)
	if s ~= nil then
		return string.gsub(s, "^.......", "")
	else
		return nil
	end
end

function keys.print(msg)
	print("\127ffffffKeys: "..msg)
end

function keys.trigger_key_info_refresh(refresh_key_id)
	keys.key_refreshes_needed_count = 0
	for i=1, GetNumKeysInKeychain() do
		local ki = {}
		ki.id, ki.desc, ki.owner_key_id, ki.time, ki.u1, ki.u2, ki.active = GetKeyInfo(i)
		if ( (refresh_key_id and ki.id == refresh_key_id) or                                   -- key_id to be refreshed was specified
			   (keys.ui.manager.selected_key_id and keys.ui.manager.selected_key_id == ki.id) or -- key is selected in manager
			   (keys.ui.copier.selected_key_id and keys.ui.copier.selected_key_id == ki.id)      -- key is selected in copier
			 )
		then
			-- trigger key refresh (needed for updating member list)
			Keychain.list(ki.id)
			keys.key_refreshes_needed_count = keys.key_refreshes_needed_count + 1
		end
	end
end

function keys.update_ui_after_key_change()
	-- copier
	keys.ui.copier.activate_buttons()
	keys.ui.copier.update_member_list()

	-- manager
	keys.ui.manager.update_member_list()
end

function keys.store_refreshed_key_infos()
	keys.infos = {}

	local owner_key_ids = {}

	for i=1, GetNumKeysInKeychain() do 
		local ki = {}
		ki.id, ki.desc, ki.owner_key_id, ki.time, ki.u1, ki.u2, ki.active = GetKeyInfo(i)
		ki.desc = string.sub(ki.desc, 0, 75) -- limit key 'name' size to 75 chars
		-- insert user names
		ki.members = {}
		if ki.u2 ~= nil then
			for _,userid in pairs(ki.u2) do
				local player_name = GetPlayerName(userid)
				if string.match(player_name, "^[^\*].*") then
					local faction = GetPlayerFaction(userid) or 0
					local print_name = rgbtohex(FactionColor_RGB[faction])
					print_name = print_name .. player_name
					table.insert(ki.members, { id = userid,
					                           owner_key_id = ki.owner_key_id,
					                           name = player_name,
					                           active = ki.active,
					                           print_name = print_name
					                         }
					            )
				end
			end
		end
		table.sort (ki.members, function (a, b) return string.lower(a.name) < string.lower(b.name) end )

		-- store owner key ids
		if ki.owner_key_id == false then
			owner_key_ids[ki.id] = true
		end

		-- store info
		table.insert(keys.infos, ki)
	end
	table.sort (keys.infos, function (a, b) return string.lower(a.desc) < string.lower(b.desc) end )

	-- mark keys that you own
	for _,ki in pairs(keys.infos) do
		if ki.owner_key_id == false or owner_key_ids[ki.owner_key_id] ~= nil then
			ki.is_yours = true
		else
			ki.is_yours = false
		end
	end
end

-- supress spammy like messages
local orig_chatreceiver = chatreceiver.OnEvent
function chatreceiver:OnEvent(event, data)
	if event == 'CHAT_MSG_SERVER' and (
		data.msg == 'Key info received.' 
		or
		string.match(data.msg, "User Key given to .*")
		or
		data.msg == "Error: That player already has that key."
		or
		string.match(data.msg, "User Key taken from .*") )
	then
		return
	end
	orig_chatreceiver(chatreceiver, event, data)
end

-- catch the ball
function keys:OnEvent(event, data)
	if keys.running == false then 
		return 
	end

	if event == "CHAT_MSG_SERVER" then
		if data.msg == 'Key info received.' then
			keys.wait_until_all_keys_are_triggered(keys.update_ui_after_key_change)
			keys.key_refreshes_needed_count = keys.key_refreshes_needed_count - 1
		end

		if string.match(data.msg, "User Key given to .*") or
		   data.msg == "Error: That player already has that key." or
		   string.match(data.msg, "User Key taken from .*")
		then
			-- continue 2 secs after the last "key given"
			keys.refresh_key_info_timer:SetTimeout(2000,
				function()
					keys.trigger_key_info_refresh()
				end
			)
		end

		if data.msg == "Error: You do not have that Owner Key." or
		   data.msg == "Error: You cannot take a User Key from someone who has an Owner Key." or
		   data.msg == "Error: That character does not exist." or
		   string.match(data.msg, "Error: That player .* does not exist.")
		then
			-- continue updating ui on error
			keys.update_ui_after_key_change()
		end

	end
end
RegisterEvent(keys, "CHAT_MSG_SERVER")

function keys.wait_until_all_keys_are_triggered(next_job)
	if keys.key_refreshes_needed_count > 0 then
		-- Call this again. There is more to trigger...
		keys.start_timer:SetTimeout(500, function() keys.wait_until_all_keys_are_triggered(next_job) end)
	else
		keys.store_refreshed_key_infos()
		next_job()
	end
end

function keys.start_func(_, args)
	keys.running = true
	keys.ui.manager.selected_key_id = nil
	keys.ui.copier.selected_key_id = nil
	keys.ui.copier.selected_owner_key_id = nil

	keys.print("loading, please wait")

	keys.trigger_key_info_refresh() -- this triggers refresh for all keys
	keys.wait_until_all_keys_are_triggered(keys.ui.create_dialog) -- parameter is function ptr for "next job"
end
RegisterUserCommand("keys", keys.start_func)
