function keys.ui.copier.deactivate_buttons()
	keys.ui.copier.clear_selected_clipboard_button.active = "NO"
	keys.ui.copier.clear_clipboard_button.active = "NO"
	keys.ui.copier.copy_selected_to_clipboard_button.active = "NO"
	keys.ui.copier.copy_all_to_clipboard_button.active = "NO"
	keys.ui.copy_selected_from_clipboard_button.active = "NO"
	keys.ui.copy_all_from_clipboard_button.active = "NO"
	keys.ui.copier.clear_all_key_members_button.active = "NO"
	keys.ui.copier.clear_selected_key_members_button.active = "NO"
end

function keys.ui.copier.activate_buttons()
	keys.ui.copier.clear_selected_clipboard_button.active = "YES"
	keys.ui.copier.clear_clipboard_button.active = "YES"
	keys.ui.copier.copy_selected_to_clipboard_button.active = "YES"
	keys.ui.copier.copy_all_to_clipboard_button.active = "YES"
	keys.ui.copy_selected_from_clipboard_button.active = "YES"
	keys.ui.copy_all_from_clipboard_button.active = "YES"
	keys.ui.copier.clear_all_key_members_button.active = "YES"
	keys.ui.copier.clear_selected_key_members_button.active = "YES"
end

function keys.ui.copier.generate_copier_tab(copier_tab)
	keys.clipboard_members = {}
	keys.ui.copier.selected_key_id = nil
	keys.ui.copier.selected_owner_key_id = nil
	keys.ui.copier.selected_key_members = nil

	-- a_hbox elements (1st column)
	local key_list = iup.list {"<select key>", dropdown = "YES", expand = "HORIZONTAL", size = "50%"}
	local key_list_i = 2
	for _,ki in pairs(keys.infos) do
		if ki.is_yours then
			local color_str = ""
			local owner_key_id_str = ki.owner_key_id
			if ki.owner_key_id == false then
				owner_key_id_str = ""
				color_str = "\127FFD559"
			else
				if ki.active then
					color_str = "\1275BB349"
				else
					color_str = "\127dddddd"
				end
			end
			key_list[key_list_i] = color_str..ki.desc.." \127aaaaaa(k_id="..ki.id..",o_id="..owner_key_id_str..")"
			key_list_i = key_list_i + 1
		end
	end
	keys.ui.copier.key_member_list = iup.list {expand="YES",readonly="YES",multiple="YES"}
	function key_list:action (text, item, state)
		keys.ui.copier.deactivate_buttons()

		while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list

		local key_id
		key_id = string.match(text, "k_id=([0-9]+)")
		if key_id ~= nil then
			keys.ui.copier.key_member_list[1] = "Loading"
			keys.trigger_key_info_refresh(key_id)
			keys.ui.copier.selected_key_id = key_id
		else
			keys.ui.copier.selected_key_id = nil
		end

		keys.ui.copier.selected_owner_key_id = nil
		local owner_key_id
		owner_key_id = string.match(text, "o_id=([0-9]+)")
		if owner_key_id ~= nil then
			keys.ui.copier.selected_owner_key_id = owner_key_id
		end
	end

	keys.ui.copier.clear_selected_key_members_button = iup.button{title="Clear selected key members",expand="HORIZONTAL", fgcolor="92 158 204"}
	function keys.ui.copier.clear_selected_key_members_button:action()
		if keys.ui.copier.selected_key_id == nil then return end

		if keys.ui.copier.selected_owner_key_id == nil then
			keys.print("Whoops, you'd better not to try clear owner members :)")
			return
		end

		local ui_refresh_needed = false
		-- iterate (+revoke) all selections 
		for i = 1, #keys.ui.copier.key_member_list.value do
			local c = keys.ui.copier.key_member_list.value:sub(i,i)
			if c == '+' then
				local selected_name = keys.ui.copier.key_member_list[i]
				if selected_name ~= nil then
					local name = keys.remove_color(selected_name)
					if name == GetPlayerName() then
						keys.print("not revoking from you...")
					else
						Keychain.revokekey(name, keys.ui.copier.selected_key_id)
						ui_refresh_needed = true
					end
				end
			end
		end
		if ui_refresh_needed then
			-- set UI to progress state
			keys.ui.copier.deactivate_buttons()
			while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
			keys.ui.copier.key_member_list[1] = "Reloading"
		end
	end

	keys.ui.copier.clear_all_key_members_button = iup.button{title="Clear all key members",expand="HORIZONTAL"}
	function keys.ui.copier.clear_all_key_members_button:action()
		if keys.ui.copier.selected_key_id == nil then return end

		if keys.ui.copier.selected_owner_key_id == nil then
			keys.print("Whoops, you'd better not to try clear owner members :)")
			return
		end

		-- set UI to progress state
		keys.ui.copier.deactivate_buttons()
		while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
		keys.ui.copier.key_member_list[1] = "Reloading"
		-- revoke keys
		local my_name = GetPlayerName()
		for _,m in pairs(keys.ui.copier.selected_key_members) do
			if m.name == my_name then
				keys.print("not revoking from you...")
			else
				Keychain.revokekey(m.name, keys.ui.copier.selected_key_id)
			end
			-- revoking keys won't update instantly, so ..
			-- continue process after all keys are processed ..
			-- .. check OnEvent()
		end
	end

	-- b_hbox elements (2nd column)
	keys.ui.copier.copy_selected_to_clipboard_button = iup.button{title="Selected",expand="HORIZONTAL", fgcolor="92 158 204"}
	keys.ui.copier.copy_all_to_clipboard_button = iup.button{title="All",expand="HORIZONTAL"}
	keys.ui.copy_selected_from_clipboard_button = iup.button{title="Selected",expand="HORIZONTAL", fgcolor="92 158 204"}
	keys.ui.copy_all_from_clipboard_button = iup.button{title="All",expand="HORIZONTAL"}

	-- c_hbox elements (3rd column)
	local clipboard_member_list = iup.list {expand="YES",multiple="YES"}
	keys.ui.copier.clear_clipboard_button = iup.button{title="Clear all", expand="HORIZONTAL"}	
	function keys.ui.copier.clear_clipboard_button:action()
		while(clipboard_member_list[1]) do clipboard_member_list[1] = nil end -- clear list
		keys.clipboard_members = {}
	end
	keys.ui.copier.clear_selected_clipboard_button = iup.button{title="Clear selected", expand="HORIZONTAL", fgcolor="92 158 204"}	
	function keys.ui.copier.clear_selected_clipboard_button:action()
		local updated_clipboard_members = {}
		for i = 1, #clipboard_member_list.value do
			local c = clipboard_member_list.value:sub(i,i)
			if c == '-' then -- '-' = not selected
				local selected_name = clipboard_member_list[i]
				if selected_name ~= nil then
					local name = keys.remove_color(selected_name)
					for _,m in pairs(keys.clipboard_members) do
						if name == m.name then
							local exists = false
							for _,ucm in pairs(updated_clipboard_members) do
								if ucm.name == name then
									exists = true
								end
							end
							if exists == false then
								table.insert(updated_clipboard_members, m)
							end
						end
					end
				end
			end
		end
		keys.clipboard_members = updated_clipboard_members
		--refresh ui
		while(clipboard_member_list[1]) do clipboard_member_list[1] = nil end -- clear list
		local clipboard_member_list_i = 1
		for _,m in pairs(keys.clipboard_members) do
			clipboard_member_list[clipboard_member_list_i] = m.print_name
			clipboard_member_list_i = clipboard_member_list_i + 1
		end
	end

	-- ** copy buttons actions **
	-- copy selected -> clipboard
	function keys.ui.copier.copy_selected_to_clipboard_button:action()
		if keys.ui.copier.selected_key_id == nil then return end
		for i = 1, #keys.ui.copier.key_member_list.value do
			local c = keys.ui.copier.key_member_list.value:sub(i,i)
			if c == '+' then
				local selected_item = keys.ui.copier.key_member_list[i]
				if selected_item ~= nil then
					for _,m in pairs(keys.ui.copier.selected_key_members) do
						if m.print_name == selected_item then
							local exists = false
							for _,cm in pairs(keys.clipboard_members) do
								if cm.name == m.name then
									exists = true
								end
							end
							if exists == false then
								table.insert(keys.clipboard_members, m)
							end
						end
					end
				end
			end
		end
		--refresh ui
		while(clipboard_member_list[1]) do clipboard_member_list[1] = nil end -- clear list
		local clipboard_member_list_i = 1
		for _,m in pairs(keys.clipboard_members) do
			clipboard_member_list[clipboard_member_list_i] = m.print_name
			clipboard_member_list_i = clipboard_member_list_i + 1
		end
	end
	-- copy all -> clipboard
	function keys.ui.copier.copy_all_to_clipboard_button:action()
		if keys.ui.copier.selected_key_id == nil then return end
		for _,m in pairs(keys.ui.copier.selected_key_members) do
			local exists = false
			for _,cm in pairs(keys.clipboard_members) do
				if cm.name == m.name then
					exists = true
				end
			end
			if exists == false then
				table.insert(keys.clipboard_members, m)
			end
		end
		--refresh ui
		while(clipboard_member_list[1]) do clipboard_member_list[1] = nil end -- clear list
		local clipboard_member_list_i = 1
		for _,m in pairs(keys.clipboard_members) do
			clipboard_member_list[clipboard_member_list_i] = m.print_name
			clipboard_member_list_i = clipboard_member_list_i + 1
		end
	end
	-- copy selected from clipboard
	function keys.ui.copy_selected_from_clipboard_button:action()
		if keys.ui.copier.selected_key_id == nil then return end
		local ui_refresh_needed = false
		for i = 1, #clipboard_member_list.value do
			local c = clipboard_member_list.value:sub(i,i)
			if c == '+' then
				local selected_name = clipboard_member_list[i]
				if selected_name ~= nil then
					local name = keys.remove_color(selected_name)
					-- give key
					Keychain.giveuserkey(name, keys.ui.copier.selected_key_id)
					ui_refresh_needed = true
				end
			end
		end
		if ui_refresh_needed then
			-- set UI to progress state
			keys.ui.copier.deactivate_buttons()
			while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
			keys.ui.copier.key_member_list[1] = "Reloading"
		end
		-- giving keys won't update instantly, so ..
		-- continue process after all keys are given ..
		-- .. check OnEvent()
	end
	-- copy all from clipboard
	function keys.ui.copy_all_from_clipboard_button:action()
		if keys.ui.copier.selected_key_id == nil then return end
		-- set UI to progress state
		keys.ui.copier.deactivate_buttons()
		while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
		keys.ui.copier.key_member_list[1] = "Reloading"
		-- give keys
		for _,m in pairs(keys.clipboard_members) do
			Keychain.giveuserkey(m.name, keys.ui.copier.selected_key_id)
			-- giving keys won't update instantly, so ..
			-- continue process after all keys are given ..
			-- .. check OnEvent()
		end
		-- giving keys won't update instantly, so ..
		-- continue process after all keys are given ..
		-- .. check OnEvent()
	end

	-- copier tab ui
	local a_vbox = iup.vbox { iup.label{title="Key Name"},
	                          key_list,
	                          iup.fill{},
	                          iup.label{title="Members"},
	                          keys.ui.copier.key_member_list,
	                          iup.hbox {
	                            keys.ui.copier.clear_selected_key_members_button,
	                            keys.ui.copier.clear_all_key_members_button,
	                            gap=15
	                          }
	                        }
	local b_vbox = iup.vbox { iup.fill{}, iup.label{title="Copy user keys"},
	                          iup.vbox{
	                            iup.label{title="To clipboard ->"},
	                            iup.hbox{
	                              keys.ui.copier.copy_selected_to_clipboard_button,
	                              keys.ui.copier.copy_all_to_clipboard_button,
	                              gap=15
	                            }
	                          },
	                         iup.vbox{
	                           iup.label{title="<- From clipboard"},
	                           iup.hbox{
	                             keys.ui.copy_selected_from_clipboard_button,
	                             keys.ui.copy_all_from_clipboard_button,
	                             gap=15
	                           }
	                         },
	                         iup.fill{},
	                         alignment="ACENTER",
	                         gap=15}
	local c_vbox = iup.vbox { iup.label{title="Clipboard"},
	                          clipboard_member_list,
	                          iup.hbox{
	                            keys.ui.copier.clear_selected_clipboard_button,
	                            keys.ui.copier.clear_clipboard_button,
	                            gap=15
	                          }
	}
	local copier_box = iup.hbox {a_vbox, b_vbox, c_vbox}
	copier_tab:append(copier_box)
end

function keys.ui.copier.update_member_list()
	if keys.ui.copier.selected_key_id == nil then return end

	while(keys.ui.copier.key_member_list[1]) do keys.ui.copier.key_member_list[1] = nil end -- clear list
	for _,ki in pairs(keys.infos) do
		if ki.id == keys.ui.copier.selected_key_id then
			keys.ui.copier.selected_key_members = {}
			local key_member_list_i = 1
			for _,m in pairs(ki.members) do
				table.insert(keys.ui.copier.selected_key_members, m)
				keys.ui.copier.key_member_list[key_member_list_i] = m.print_name
				key_member_list_i = key_member_list_i + 1
			end
			-- update copy/clear button states
			if ki.is_yours then
				keys.ui.copy_selected_from_clipboard_button.active = "YES"
				keys.ui.copy_all_from_clipboard_button.active = "YES"
				keys.ui.copier.clear_all_key_members_button.active = "YES"
				keys.ui.copier.clear_selected_key_members_button.active = "YES"
			else
				keys.ui.copy_selected_from_clipboard_button.active = "NO"
				keys.ui.copy_all_from_clipboard_button.active = "NO"
				keys.ui.copier.clear_all_key_members_button.active = "NO"
				keys.ui.copier.clear_selected_key_members_button.active = "NO"
			end
		end
	end
end
