function keys.ui.manager.generate_manager_tab(manager_tab)

	-- key list
	local key_list = iup.list {"<select key>", dropdown = "YES", expand = "HORIZONTAL"}
	local key_list_i = 2
	for _,ki in pairs(keys.infos) do
		local color_str = ""
		local owner_key_id_str = ki.owner_key_id
		if ki.is_yours then
			color_str = "\127dddddd"
			if ki.owner_key_id == false then
				owner_key_id_str = ""
				color_str = "\127FFD559"
			else
				if ki.active then
					color_str = "\1275BB349"
				end
			end
			key_list[key_list_i] = color_str..ki.desc.." \127aaaaaa(k_id="..ki.id..",o_id="..owner_key_id_str..")"
			key_list_i = key_list_i + 1
		end
	end
	-- key members
	keys.ui.manager.key_member_list = iup.list {expand="YES",readonly="YES",multiple="YES"}
	function key_list:action (text, item, state)
		while(keys.ui.manager.key_member_list[1]) do keys.ui.manager.key_member_list[1] = nil end -- clear list
		local key_id
		key_id = string.match(text, "k_id=([0-9]+)")
		if key_id ~= nil then
			keys.ui.manager.key_member_list[1] = "Loading"
			keys.trigger_key_info_refresh(key_id)
			keys.ui.manager.selected_key_id = key_id
		else
			keys.ui.manager.selected_key_id = nil
		end
	end

	-- revoke button
	keys.ui.manager.revoke_selected_button = iup.button{title="Revoke selected",expand="HORIZONTAL"}
	function keys.ui.manager.revoke_selected_button:action()
		if keys.ui.manager.selected_key_id == nil then return end
		for i = 1, #keys.ui.manager.key_member_list.value do
			local c = keys.ui.manager.key_member_list.value:sub(i,i)
			if c == '+' then
				local selected_name = keys.ui.manager.key_member_list[i]
				if selected_name == nil then return end
				local name = keys.remove_color(selected_name)
				if name == GetPlayerName() then
					keys.print("not revoking from you...")
				else
					Keychain.revokekey(name, keys.ui.manager.selected_key_id)
				end
			end
		end
		while(keys.ui.manager.key_member_list[1]) do keys.ui.manager.key_member_list[1] = nil end -- clear list
		keys.ui.manager.key_member_list[1] = "Reloading"
	end

	-- give key
	keys.ui.manager.text_before_ac = ""
	keys.ui.manager.seen_player_names = GetListOfSeenNames()
	keys.ui.manager.name_field = iup.text{visiblecolumns="30", expand="HORIZONTAL"}
	keys.ui.manager.complete_button = iup.button{title="-->        "}
	function keys.ui.manager.complete_button:action()
		local text = keys.ui.manager.name_field['value']
		if #text == 0 or string.match(text, '^\*.*') ~= nil then return end
		if keys.ui.manager.autocomplete_index == nil or
		   keys.ui.manager.autocomplete_index == 1 or
		   text ~= keys.ui.manager.autocomplete_list[keys.ui.manager.autocomplete_index]
		then
			keys.ui.manager.complete_button['title'] = "--> (cycle)"
			keys.ui.manager.text_before_ac = text
			keys.ui.manager.autocomplete_list = {}
			keys.ui.manager.autocomplete_index = 1
			keys.ui.manager.autocomplete_last_index = 1
			table.insert(keys.ui.manager.autocomplete_list, text)
			for _,name in ipairs(keys.ui.manager.seen_player_names) do
				if string.sub(string.lower(name), 1, #text) == string.lower(text) then
					table.insert(keys.ui.manager.autocomplete_list, name)
					keys.ui.manager.autocomplete_last_index = keys.ui.manager.autocomplete_last_index + 1
				end
			end
		end
		if keys.ui.manager.autocomplete_index < keys.ui.manager.autocomplete_last_index then
			keys.ui.manager.autocomplete_index = keys.ui.manager.autocomplete_index + 1
		elseif keys.ui.manager.autocomplete_index == keys.ui.manager.autocomplete_last_index then
			keys.ui.manager.autocomplete_index = 1
			keys.ui.manager.complete_button['title'] = "-->        "
		end
		keys.ui.manager.name_field['value'] = keys.ui.manager.autocomplete_list[keys.ui.manager.autocomplete_index]
	end
	local give_button = iup.button{title="Give user key", expand="HORIZONTAL"}
	function give_button:action()
		if keys.ui.manager.selected_key_id == nil then return end
		local name = keys.ui.manager.name_field['value']
		if #name == 0 then return nil end
		while(keys.ui.manager.key_member_list[1]) do keys.ui.manager.key_member_list[1] = nil end -- clear list
		keys.ui.manager.key_member_list[1] = "Reloading"
		Keychain.giveuserkey(name, keys.ui.manager.selected_key_id)
	end
	local give_box = iup.vbox{
		iup.hbox{
			iup.label{title="Give user-key"}
		},
		iup.hbox{
			iup.label{title="Player name: "},
			keys.ui.manager.name_field, keys.ui.manager.complete_button
		},
		iup.hbox{
			give_button
		}
	}

	-- merge
	local box = iup.vbox{ iup.label{title="Key Name"},
	                      key_list, 
	                      iup.label{title="Members"},
	                      keys.ui.manager.key_member_list, 
	                      iup.fill{size="%5"},
	                      iup.hbox{
	                        give_box,
	                        iup.fill{size="%15"},
	                        iup.vbox{
	                          iup.label{title="Revoke user-key"},
	                          keys.ui.manager.revoke_selected_button
	                        }
	                      }
	                    }
	manager_tab:append(box)
end

function keys.ui.manager.update_member_list()
	if keys.ui.manager.selected_key_id == nil then return end

	while(keys.ui.manager.key_member_list[1]) do keys.ui.manager.key_member_list[1] = nil end -- clear list
	keys.ui.manager.selected_key_members = {}
	if keys.ui.manager.selected_key_id ~= nil then
		for _,ki in pairs(keys.infos) do
			if ki.id == keys.ui.manager.selected_key_id then
				local key_member_list_i = 1
				for _,m in pairs(ki.members) do
					table.insert(keys.ui.manager.selected_key_members, m)
					keys.ui.manager.key_member_list[key_member_list_i] = m.print_name
					key_member_list_i = key_member_list_i + 1
				end
			end
		end
	end
end
